@@ -1,26 +1,22 @@
-#ifndef PETSCTAO_H
-#define PETSCTAO_H
+#pragma once
 
 #include <petscsnes.h>
 
 /* SUBMANSEC = Tao */
 
-PETSC_EXTERN PetscErrorCode VecFischer(Vec, Vec, Vec, Vec, Vec);
-PETSC_EXTERN PetscErrorCode VecSFischer(Vec, Vec, Vec, Vec, PetscReal, Vec);
-PETSC_EXTERN PetscErrorCode MatDFischer(Mat, Vec, Vec, Vec, Vec, Vec, Vec, Vec, Vec);
 PETSC_EXTERN PetscErrorCode MatDSFischer(Mat, Vec, Vec, Vec, Vec, PetscReal, Vec, Vec, Vec, Vec, Vec);
 PETSC_EXTERN PetscErrorCode TaoSoftThreshold(Vec, PetscReal, PetscReal, Vec);
 
 /*E
-  TaoSubsetType - Type representing the way TAO handles active sets
+  TaoSubsetType - Type representing the way the `Tao` solvers handle active sets
 
   Values:
-+ `TAO_SUBSET_SUBVEC` - Tao uses `MatCreateSubMatrix()` and `VecGetSubVector()`
-. `TAO_SUBSET_MASK` - Matrices are zeroed out corresponding to active set entries
++ `TAO_SUBSET_SUBVEC`     - Tao uses `MatCreateSubMatrix()` and `VecGetSubVector()`
+. `TAO_SUBSET_MASK`       - Matrices are zeroed out corresponding to active set entries
 - `TAO_SUBSET_MATRIXFREE` - Same as `TAO_SUBSET_MASK` but it can be applied to matrix-free operators
 
   Options database Key:
-. -different_hessian - Tao will use a copy of the Hessian operator for masking.  By default TAO will directly alter the Hessian operator.
+. -different_hessian - `Tao` will use a copy of the Hessian operator for masking.  By default `Tao` will directly alter the Hessian operator.
 
   Level: intermediate
 
@@ -34,16 +30,19 @@ typedef enum {
 PETSC_EXTERN const char *const TaoSubsetTypes[];
 
 /*S
-     Tao - Abstract PETSc object that manages nonlinear optimization solves
+   Tao - Abstract PETSc object that manages optimization solvers.
 
    Level: advanced
 
+   Note:
+   `Tao` is the object, while TAO, which stands for Toolkit for Advanced Optimization, is the software package.
+
 .seealso: [](doc_taosolve), [](ch_tao), `TaoCreate()`, `TaoDestroy()`, `TaoSetType()`, `TaoType`
 S*/
 typedef struct _p_Tao *Tao;
 
 /*E
-     TaoADMMUpdateType - Determine spectral penalty update routine for Lagrange augmented term for `TAOADMM`.
+  TaoADMMUpdateType - Determine the spectral penalty update routine for the Lagrange augmented term for `TAOADMM`.
 
   Level: advanced
 
@@ -57,7 +56,7 @@ typedef enum {
 PETSC_EXTERN const char *const TaoADMMUpdateTypes[];
 
 /*MC
-     TAO_ADMM_UPDATE_BASIC - Use same spectral penalty set at the beginning. No update
+  TAO_ADMM_UPDATE_BASIC - Use same spectral penalty set at the beginning. This never performs an update to the penalty
 
   Level: advanced
 
@@ -68,7 +67,7 @@ PETSC_EXTERN const char *const TaoADMMUpdateTypes[];
 M*/
 
 /*MC
-     TAO_ADMM_UPDATE_ADAPTIVE - Adaptively update spectral penalty
+  TAO_ADMM_UPDATE_ADAPTIVE - Adaptively update the spectral penalty
 
   Level: advanced
 
@@ -79,18 +78,18 @@ M*/
 M*/
 
 /*MC
-     ADMM_UPDATE_ADAPTIVE_RELAXED - Adaptively update spectral penalty, and relaxes parameter update
+  ADMM_UPDATE_ADAPTIVE_RELAXED - Adaptively update spectral penalty, and relaxes parameter update
 
   Level: advanced
 
   Note:
-  With adaptive spectral penalty update, it also relaxes x vector update by a factor.
+  With adaptive spectral penalty update, it also relaxes the `x` vector update by a factor.
 
 .seealso: [](ch_tao), `Tao`, `TaoADMMSetUpdateType()`, `TAO_ADMM_UPDATE_BASIC`, `TAO_ADMM_UPDATE_ADAPTIVE`
 M*/
 
 /*E
-     TaoADMMRegularizerType - Determine regularizer routine - either user provided or soft threshold for `TAOADMM`
+  TaoADMMRegularizerType - Determine regularizer routine - either user provided or soft threshold for `TAOADMM`
 
   Level: advanced
 
@@ -126,11 +125,11 @@ M*/
 M*/
 
 /*E
-     TaoALMMType - Determine the augmented Lagrangian formulation used in the `TAOALMM` subproblem.
+   TaoALMMType - Determine the augmented Lagrangian formulation used in the `TAOALMM` subproblem.
 
    Values:
 +  `TAO_ALMM_CLASSIC` - classic augmented Lagrangian definition including slack variables for inequality constraints
--  `TAO_ALMM_PHR`     - Powell-Hestenes-Rockafellar formulation without slack variables, uses pointwise min() for inequalities
+-  `TAO_ALMM_PHR`     - Powell-Hestenes-Rockafellar formulation without slack variables, uses pointwise `min()` for inequalities
 
   Level: advanced
 
@@ -142,23 +141,63 @@ typedef enum {
 } TaoALMMType;
 PETSC_EXTERN const char *const TaoALMMTypes[];
 
-/*J
-        TaoType - String with the name of a `Tao` method
+/*E
+  TaoBNCGType - Determine the conjugate gradient update formula used in the `TAOBNCG` algorithm.
 
   Values:
-+    `TAONLS` - nls Newton's method with line search for unconstrained minimization
-.    `TAONTR` - ntr Newton's method with trust region for unconstrained minimization
-.    `TAONTL` - ntl Newton's method with trust region, line search for unconstrained minimization
-.    `TAOLMVM` - lmvm Limited memory variable metric method for unconstrained minimization
-.    `TAOCG` - cg Nonlinear conjugate gradient method for unconstrained minimization
-.    `TAONM` - nm Nelder-Mead algorithm for derivate-free unconstrained minimization
-.    `TAOTRON` - tron Newton Trust Region method for bound constrained minimization
-.    `TAOGPCG` - gpcg Newton Trust Region method for quadratic bound constrained minimization
-.    `TAOBLMVM` - blmvm Limited memory variable metric method for bound constrained minimization
-.    `TAOLCL` - lcl Linearly constrained Lagrangian method for pde-constrained minimization
--    `TAOPOUNDERS` - Pounders Model-based algorithm for nonlinear least squares
++  `TAO_BNCG_GD`         - basic gradient descent, no CG update
+.  `TAO_BNCG_PCGD`       - preconditioned/scaled gradient descent
+.  `TAO_BNCG_HS`         - Hestenes-Stiefel
+.  `TAO_BNCG_FR`         - Fletcher-Reeves
+.  `TAO_BNCG_PRP`        - Polak-Ribiere-Polyak (PRP)
+.  `TAO_BNCG_PRP_PLUS`   - Polak-Ribiere-Polyak "plus" (PRP+)
+.  `TAO_BNCG_DY`         - Dai-Yuan
+.  `TAO_BNCG_HZ`         - Hager-Zhang (CG_DESCENT 5.3)
+.  `TAO_BNCG_DK`         - Dai-Kou (2013)
+.  `TAO_BNCG_KD`         - Kou-Dai (2015)
+.  `TAO_BNCG_SSML_BFGS`  - Self-Scaling Memoryless BFGS (Perry-Shanno)
+.  `TAO_BNCG_SSML_DFP`   - Self-Scaling Memoryless DFP
+-  `TAO_BNCG_SSML_BRDN`  - Self-Scaling Memoryless (Symmetric) Broyden
 
-       Level: beginner
+  Level: advanced
+
+.seealso: `Tao`, `TAOBNCG`, `TaoBNCGSetType()`, `TaoBNCGGetType()`
+E*/
+
+typedef enum {
+  TAO_BNCG_GD,
+  TAO_BNCG_PCGD,
+  TAO_BNCG_HS,
+  TAO_BNCG_FR,
+  TAO_BNCG_PRP,
+  TAO_BNCG_PRP_PLUS,
+  TAO_BNCG_DY,
+  TAO_BNCG_HZ,
+  TAO_BNCG_DK,
+  TAO_BNCG_KD,
+  TAO_BNCG_SSML_BFGS,
+  TAO_BNCG_SSML_DFP,
+  TAO_BNCG_SSML_BRDN
+} TaoBNCGType;
+PETSC_EXTERN const char *const TaoBNCGTypes[];
+
+/*J
+  TaoType - String with the name of a `Tao` method
+
+  Values:
++ `TAONLS`      - nls Newton's method with line search for unconstrained minimization
+. `TAONTR`      - ntr Newton's method with trust region for unconstrained minimization
+. `TAONTL`      - ntl Newton's method with trust region, line search for unconstrained minimization
+. `TAOLMVM`     - lmvm Limited memory variable metric method for unconstrained minimization
+. `TAOCG`       - cg Nonlinear conjugate gradient method for unconstrained minimization
+. `TAONM`       - nm Nelder-Mead algorithm for derivate-free unconstrained minimization
+. `TAOTRON`     - tron Newton Trust Region method for bound constrained minimization
+. `TAOGPCG`     - gpcg Newton Trust Region method for quadratic bound constrained minimization
+. `TAOBLMVM`    - blmvm Limited memory variable metric method for bound constrained minimization
+. `TAOLCL`      - lcl Linearly constrained Lagrangian method for pde-constrained minimization
+- `TAOPOUNDERS` - Pounders Model-based algorithm for nonlinear least squares
+
+  Level: beginner
 
 .seealso: [](doc_taosolve), [](ch_tao), `Tao`, `TaoCreate()`, `TaoSetType()`
 J*/
@@ -205,29 +244,29 @@ PETSC_EXTERN PetscFunctionList TaoList;
     TaoConvergedReason - reason a `Tao` optimizer was said to have converged or diverged
 
    Values:
-+  `TAO_CONVERGED_GATOL` - ||g(X)|| < gatol
-.  `TAO_CONVERGED_GRTOL` - ||g(X)|| / f(X)  < grtol
-.  `TAO_CONVERGED_GTTOL` - ||g(X)|| / ||g(X0)|| < gttol
-.  `TAO_CONVERGED_STEPTOL` - step size smaller than tolerance
-.  `TAO_CONVERGED_MINF` - F < F_min
-.  `TAO_CONVERGED_USER` - the user indicates the optimization has succeeded
-.  `TAO_DIVERGED_MAXITS` - the maximum number of iterations allowed has been achieved
-.  `TAO_DIVERGED_NAN` - not a number appeared in the computations
-.  `TAO_DIVERGED_MAXFCN` - the maximum number of function evaluations has been computed
-.  `TAO_DIVERGED_LS_FAILURE` - a linesearch failed
++  `TAO_CONVERGED_GATOL`       - $||g(X)|| < gatol$
+.  `TAO_CONVERGED_GRTOL`       - $||g(X)|| / f(X)  < grtol$
+.  `TAO_CONVERGED_GTTOL`       - $||g(X)|| / ||g(X0)|| < gttol$
+.  `TAO_CONVERGED_STEPTOL`     - step size smaller than tolerance
+.  `TAO_CONVERGED_MINF`        - $F < F_min$
+.  `TAO_CONVERGED_USER`        - the user indicates the optimization has succeeded
+.  `TAO_DIVERGED_MAXITS`       - the maximum number of iterations allowed has been achieved
+.  `TAO_DIVERGED_NAN`          - not a number appeared in the computations
+.  `TAO_DIVERGED_MAXFCN`       - the maximum number of function evaluations has been computed
+.  `TAO_DIVERGED_LS_FAILURE`   - a linesearch failed
 .  `TAO_DIVERGED_TR_REDUCTION` - trust region failure
-.  `TAO_DIVERGED_USER` - the user has indicated the optimization has failed
--  `TAO_CONTINUE_ITERATING` - the optimization is still running, `TaoSolve()`
+.  `TAO_DIVERGED_USER`         - the user has indicated the optimization has failed
+-  `TAO_CONTINUE_ITERATING`    - the optimization is still running, `TaoSolve()`
 
    where
-+  X - current solution
-.  X0 - initial guess
-.  f(X) - current function value
-.  f(X*) - true solution (estimated)
-.  g(X) - current gradient
-.  its - current iterate number
-.  maxits - maximum number of iterates
-.  fevals - number of function evaluations
++  X            - current solution
+.  X0           - initial guess
+.  f(X)         - current function value
+.  f(X*)        - true solution (estimated)
+.  g(X)         - current gradient
+.  its          - current iterate number
+.  maxits       - maximum number of iterates
+.  fevals       - number of function evaluations
 -  max_funcsals - maximum number of function evaluations
 
    Level: beginner
@@ -271,6 +310,7 @@ PETSC_EXTERN PetscErrorCode TaoGetType(Tao, TaoType *);
 PETSC_EXTERN PetscErrorCode TaoSetApplicationContext(Tao, void *);
 PETSC_EXTERN PetscErrorCode TaoGetApplicationContext(Tao, void *);
 PETSC_EXTERN PetscErrorCode TaoDestroy(Tao *);
+PETSC_EXTERN PetscErrorCode TaoParametersInitialize(Tao);
 
 PETSC_EXTERN PetscErrorCode TaoSetOptionsPrefix(Tao, const char[]);
 PETSC_EXTERN PetscErrorCode TaoView(Tao, PetscViewer);
@@ -286,14 +326,6 @@ PETSC_EXTERN PetscErrorCode TaoGetSolutionStatus(Tao, PetscInt *, PetscReal *, P
 PETSC_EXTERN PetscErrorCode TaoSetConvergedReason(Tao, TaoConvergedReason);
 PETSC_EXTERN PetscErrorCode TaoSetSolution(Tao, Vec);
 PETSC_EXTERN PetscErrorCode TaoGetSolution(Tao, Vec *);
-PETSC_DEPRECATED_FUNCTION("Use TaoSetSolution() (since version 3.17)") static inline PetscErrorCode TaoSetInitialVector(Tao t, Vec v)
-{
-  return TaoSetSolution(t, v);
-}
-PETSC_DEPRECATED_FUNCTION("Use TaoGetSolution() (since version 3.17)") static inline PetscErrorCode TaoGetInitialVector(Tao t, Vec *v)
-{
-  return TaoGetSolution(t, v);
-}
 
 PETSC_EXTERN PetscErrorCode TaoSetObjective(Tao, PetscErrorCode (*)(Tao, Vec, PetscReal *, void *), void *);
 PETSC_EXTERN PetscErrorCode TaoGetObjective(Tao, PetscErrorCode (**)(Tao, Vec, PetscReal *, void *), void **);
@@ -303,26 +335,6 @@ PETSC_EXTERN PetscErrorCode TaoSetObjectiveAndGradient(Tao, Vec, PetscErrorCode
 PETSC_EXTERN PetscErrorCode TaoGetObjectiveAndGradient(Tao, Vec *, PetscErrorCode (**)(Tao, Vec, PetscReal *, Vec, void *), void **);
 PETSC_EXTERN PetscErrorCode TaoSetHessian(Tao, Mat, Mat, PetscErrorCode (*)(Tao, Vec, Mat, Mat, void *), void *);
 PETSC_EXTERN PetscErrorCode TaoGetHessian(Tao, Mat *, Mat *, PetscErrorCode (**)(Tao, Vec, Mat, Mat, void *), void **);
-PETSC_DEPRECATED_FUNCTION("Use TaoSetObjective() (since version 3.17)") static inline PetscErrorCode TaoSetObjectiveRoutine(Tao t, PetscErrorCode (*f)(Tao, Vec, PetscReal *, void *), void *c)
-{
-  return TaoSetObjective(t, f, c);
-}
-PETSC_DEPRECATED_FUNCTION("Use TaoGetGradient() (since version 3.17)") static inline PetscErrorCode TaoGetGradientVector(Tao t, Vec *v)
-{
-  return TaoGetGradient(t, v, PETSC_NULLPTR, PETSC_NULLPTR);
-}
-PETSC_DEPRECATED_FUNCTION("Use TaoSetGradient() (since version 3.17)") static inline PetscErrorCode TaoSetGradientRoutine(Tao t, PetscErrorCode (*f)(Tao, Vec, Vec, void *), void *c)
-{
-  return TaoSetGradient(t, PETSC_NULLPTR, f, c);
-}
-PETSC_DEPRECATED_FUNCTION("Use TaoSetObjectiveAndGradient() (since version 3.17)") static inline PetscErrorCode TaoSetObjectiveAndGradientRoutine(Tao t, PetscErrorCode (*f)(Tao, Vec, PetscReal *, Vec, void *), void *c)
-{
-  return TaoSetObjectiveAndGradient(t, PETSC_NULLPTR, f, c);
-}
-PETSC_DEPRECATED_FUNCTION("Use TaoSetHessian() (since version 3.17)") static inline PetscErrorCode TaoSetHessianRoutine(Tao t, Mat H, Mat P, PetscErrorCode (*f)(Tao, Vec, Mat, Mat, void *), void *c)
-{
-  return TaoSetHessian(t, H, P, f, c);
-}
 
 PETSC_EXTERN PetscErrorCode TaoSetGradientNorm(Tao, Mat);
 PETSC_EXTERN PetscErrorCode TaoGetGradientNorm(Tao, Mat *);
@@ -338,13 +350,17 @@ PETSC_EXTERN PetscErrorCode TaoSetResidualRoutine(Tao, Vec, PetscErrorCode (*)(T
 PETSC_EXTERN PetscErrorCode TaoSetResidualWeights(Tao, Vec, PetscInt, PetscInt *, PetscInt *, PetscReal *);
 PETSC_EXTERN PetscErrorCode TaoSetConstraintsRoutine(Tao, Vec, PetscErrorCode (*)(Tao, Vec, Vec, void *), void *);
 PETSC_EXTERN PetscErrorCode TaoSetInequalityConstraintsRoutine(Tao, Vec, PetscErrorCode (*)(Tao, Vec, Vec, void *), void *);
+PETSC_EXTERN PetscErrorCode TaoGetInequalityConstraintsRoutine(Tao, Vec *, PetscErrorCode (**)(Tao, Vec, Vec, void *), void **);
 PETSC_EXTERN PetscErrorCode TaoSetEqualityConstraintsRoutine(Tao, Vec, PetscErrorCode (*)(Tao, Vec, Vec, void *), void *);
+PETSC_EXTERN PetscErrorCode TaoGetEqualityConstraintsRoutine(Tao, Vec *, PetscErrorCode (**)(Tao, Vec, Vec, void *), void **);
 PETSC_EXTERN PetscErrorCode TaoSetJacobianResidualRoutine(Tao, Mat, Mat, PetscErrorCode (*)(Tao, Vec, Mat, Mat, void *), void *);
 PETSC_EXTERN PetscErrorCode TaoSetJacobianRoutine(Tao, Mat, Mat, PetscErrorCode (*)(Tao, Vec, Mat, Mat, void *), void *);
 PETSC_EXTERN PetscErrorCode TaoSetJacobianStateRoutine(Tao, Mat, Mat, Mat, PetscErrorCode (*)(Tao, Vec, Mat, Mat, Mat, void *), void *);
 PETSC_EXTERN PetscErrorCode TaoSetJacobianDesignRoutine(Tao, Mat, PetscErrorCode (*)(Tao, Vec, Mat, void *), void *);
 PETSC_EXTERN PetscErrorCode TaoSetJacobianInequalityRoutine(Tao, Mat, Mat, PetscErrorCode (*)(Tao, Vec, Mat, Mat, void *), void *);
+PETSC_EXTERN PetscErrorCode TaoGetJacobianInequalityRoutine(Tao, Mat *, Mat *, PetscErrorCode (**)(Tao, Vec, Mat, Mat, void *), void **);
 PETSC_EXTERN PetscErrorCode TaoSetJacobianEqualityRoutine(Tao, Mat, Mat, PetscErrorCode (*)(Tao, Vec, Mat, Mat, void *), void *);
+PETSC_EXTERN PetscErrorCode TaoGetJacobianEqualityRoutine(Tao, Mat *, Mat *, PetscErrorCode (**)(Tao, Vec, Mat, Mat, void *), void **);
 
 PETSC_EXTERN PetscErrorCode TaoPythonSetType(Tao, const char[]);
 PETSC_EXTERN PetscErrorCode TaoPythonGetType(Tao, const char *[]);
@@ -353,15 +369,6 @@ PETSC_EXTERN PetscErrorCode TaoShellSetSolve(Tao, PetscErrorCode (*)(Tao));
 PETSC_EXTERN PetscErrorCode TaoShellSetContext(Tao, void *);
 PETSC_EXTERN PetscErrorCode TaoShellGetContext(Tao, void *);
 
-PETSC_DEPRECATED_FUNCTION("Use TaoSetResidualRoutine() (since version 3.11)") static inline PetscErrorCode TaoSetSeparableObjectiveRoutine(Tao tao, Vec res, PetscErrorCode (*func)(Tao, Vec, Vec, void *), void *ctx)
-{
-  return TaoSetResidualRoutine(tao, res, func, ctx);
-}
-PETSC_DEPRECATED_FUNCTION("Use TaoSetResidualWeights() (since version 3.11)") static inline PetscErrorCode TaoSetSeparableObjectiveWeights(Tao tao, Vec sigma_v, PetscInt n, PetscInt *rows, PetscInt *cols, PetscReal *vals)
-{
-  return TaoSetResidualWeights(tao, sigma_v, n, rows, cols, vals);
-}
-
 PETSC_EXTERN PetscErrorCode TaoSetStateDesignIS(Tao, IS, IS);
 
 PETSC_EXTERN PetscErrorCode TaoComputeObjective(Tao, Vec, PetscReal *);
@@ -377,11 +384,6 @@ PETSC_EXTERN PetscErrorCode TaoIsObjectiveDefined(Tao, PetscBool *);
 PETSC_EXTERN PetscErrorCode TaoIsGradientDefined(Tao, PetscBool *);
 PETSC_EXTERN PetscErrorCode TaoIsObjectiveAndGradientDefined(Tao, PetscBool *);
 
-PETSC_DEPRECATED_FUNCTION("Use TaoComputeResidual() (since version 3.11)") static inline PetscErrorCode TaoComputeSeparableObjective(Tao tao, Vec X, Vec F)
-{
-  return TaoComputeResidual(tao, X, F);
-}
-
 PETSC_EXTERN PetscErrorCode TaoTestHessian(Tao);
 PETSC_EXTERN PetscErrorCode TaoComputeHessian(Tao, Vec, Mat, Mat);
 PETSC_EXTERN PetscErrorCode TaoComputeResidualJacobian(Tao, Vec, Mat, Mat);
@@ -438,23 +440,19 @@ PETSC_EXTERN PetscErrorCode TaoGetLineSearch(Tao, TaoLineSearch *);
 
 PETSC_EXTERN PetscErrorCode TaoSetConvergenceHistory(Tao, PetscReal *, PetscReal *, PetscReal *, PetscInt *, PetscInt, PetscBool);
 PETSC_EXTERN PetscErrorCode TaoGetConvergenceHistory(Tao, PetscReal **, PetscReal **, PetscReal **, PetscInt **, PetscInt *);
-PETSC_EXTERN PetscErrorCode TaoSetMonitor(Tao, PetscErrorCode (*)(Tao, void *), void *, PetscErrorCode (*)(void **));
-PETSC_EXTERN PetscErrorCode TaoCancelMonitors(Tao);
+PETSC_EXTERN PetscErrorCode TaoMonitorSet(Tao, PetscErrorCode (*)(Tao, void *), void *, PetscCtxDestroyFn *);
+PETSC_EXTERN PetscErrorCode TaoMonitorCancel(Tao);
 PETSC_EXTERN PetscErrorCode TaoMonitorDefault(Tao, void *);
-PETSC_DEPRECATED_FUNCTION("Use TaoMonitorDefault() (since version 3.9)") static inline PetscErrorCode TaoDefaultMonitor(Tao tao, void *ctx)
-{
-  return TaoMonitorDefault(tao, ctx);
-}
-PETSC_EXTERN PetscErrorCode TaoDefaultGMonitor(Tao, void *);
-PETSC_EXTERN PetscErrorCode TaoDefaultSMonitor(Tao, void *);
-PETSC_EXTERN PetscErrorCode TaoDefaultCMonitor(Tao, void *);
-PETSC_EXTERN PetscErrorCode TaoSolutionMonitor(Tao, void *);
-PETSC_EXTERN PetscErrorCode TaoResidualMonitor(Tao, void *);
-PETSC_EXTERN PetscErrorCode TaoGradientMonitor(Tao, void *);
-PETSC_EXTERN PetscErrorCode TaoStepDirectionMonitor(Tao, void *);
-PETSC_EXTERN PetscErrorCode TaoDrawSolutionMonitor(Tao, void *);
-PETSC_EXTERN PetscErrorCode TaoDrawStepMonitor(Tao, void *);
-PETSC_EXTERN PetscErrorCode TaoDrawGradientMonitor(Tao, void *);
+PETSC_EXTERN PetscErrorCode TaoMonitorGlobalization(Tao, void *);
+PETSC_EXTERN PetscErrorCode TaoMonitorDefaultShort(Tao, void *);
+PETSC_EXTERN PetscErrorCode TaoMonitorConstraintNorm(Tao, void *);
+PETSC_EXTERN PetscErrorCode TaoMonitorSolution(Tao, void *);
+PETSC_EXTERN PetscErrorCode TaoMonitorResidual(Tao, void *);
+PETSC_EXTERN PetscErrorCode TaoMonitorGradient(Tao, void *);
+PETSC_EXTERN PetscErrorCode TaoMonitorStep(Tao, void *);
+PETSC_EXTERN PetscErrorCode TaoMonitorSolutionDraw(Tao, void *);
+PETSC_EXTERN PetscErrorCode TaoMonitorStepDraw(Tao, void *);
+PETSC_EXTERN PetscErrorCode TaoMonitorGradientDraw(Tao, void *);
 PETSC_EXTERN PetscErrorCode TaoAddLineSearchCounts(Tao);
 
 PETSC_EXTERN PetscErrorCode TaoDefaultConvergenceTest(Tao, void *);
@@ -466,13 +464,56 @@ typedef struct _n_TaoMonitorDrawCtx *TaoMonitorDrawCtx;
 PETSC_EXTERN PetscErrorCode          TaoMonitorDrawCtxCreate(MPI_Comm, const char[], const char[], int, int, int, int, PetscInt, TaoMonitorDrawCtx *);
 PETSC_EXTERN PetscErrorCode          TaoMonitorDrawCtxDestroy(TaoMonitorDrawCtx *);
 
+/*E
+  TaoBRGNRegularizationType - The regularization added in the `TAOBRGN` solver.
+
+  Values:
++ TAOBRGN_REGULARIZATION_USER   - A user-defined regularizer
+. TAOBRGN_REGULARIZATION_L2PROX - $\tfrac{1}{2}\|x - x_k\|_2^$, where $x_k$ is the latest solution
+. TAOBRGN_REGULARIZATION_L2PURE - $\tfrac{1}{2}\|x\|_2^2$
+. TAOBRGN_REGULARIZATION_L1DICT - $\|D x\|_1$, where $D$ is a dictionary matrix
+- TAOBRGN_REGULARIZATION_LM     - Levenberg-Marquardt, $\tfrac{1}{2} x^T \mathrm{diag}(J^T J) x$, where $J$ is the Jacobian of the least-squares residual
+
+  Options database Key:
+. -tao_brgn_regularization_type <user,l2prox,l2pure,l1dict,lm> - one of the above regularization types
+
+  Level: advanced
+
+  Notes:
+  If `TAOBRGN_REGULARIZATION_USER`, the regularizer is set either by calling
+  `TaoBRGNSetRegularizerObjectiveAndGradientRoutine()` and
+  `TaoBRGNSetRegulazerHessianRoutine()` or by calling `TaoBRGNSetRegularizerTerm()`.
+
+  If `TAOBRGN_REGULARIZATION_L1DICT`, the dictionary matrix is set with `TaoBRGNSetDictionaryMatrix()` and the smoothing parameter of the
+  approximate $\ell_1$ norm is set with `TaoBRGNSetL1SmoothEpsilon()`.
+
+  If `TAOBRGN_REGULARIZATION_LM`, the diagonal damping vector $\mathrm{diag}(J^T J)$ can be obtained with `TaoBRGNGetDampingVector()`.
+
+.seealso: [](ch_tao), `Tao`, `TaoBRGNGetSubsolver()`, `TaoBRGNSetRegularizerWeight()`, `TaoBRGNSetL1SmoothEpsilon()`, `TaoBRGNSetDictionaryMatrix()`,
+          `TaoBRGNSetRegularizerObjectiveAndGradientRoutine()`, `TaoBRGNSetRegularizerHessianRoutine()`,
+          `TaoBRGNGetRegularizationType()`, `TaoBRGNSetRegularizationType()`
+E*/
+typedef enum {
+  TAOBRGN_REGULARIZATION_USER,
+  TAOBRGN_REGULARIZATION_L2PROX,
+  TAOBRGN_REGULARIZATION_L2PURE,
+  TAOBRGN_REGULARIZATION_L1DICT,
+  TAOBRGN_REGULARIZATION_LM,
+} TaoBRGNRegularizationType;
+
+PETSC_EXTERN const char *const TaoBRGNRegularizationTypes[];
+
 PETSC_EXTERN PetscErrorCode TaoBRGNGetSubsolver(Tao, Tao *);
+PETSC_EXTERN PetscErrorCode TaoBRGNGetRegularizationType(Tao, TaoBRGNRegularizationType *);
+PETSC_EXTERN PetscErrorCode TaoBRGNSetRegularizationType(Tao, TaoBRGNRegularizationType);
 PETSC_EXTERN PetscErrorCode TaoBRGNSetRegularizerObjectiveAndGradientRoutine(Tao, PetscErrorCode (*)(Tao, Vec, PetscReal *, Vec, void *), void *);
 PETSC_EXTERN PetscErrorCode TaoBRGNSetRegularizerHessianRoutine(Tao, Mat, PetscErrorCode (*)(Tao, Vec, Mat, void *), void *);
 PETSC_EXTERN PetscErrorCode TaoBRGNSetRegularizerWeight(Tao, PetscReal);
 PETSC_EXTERN PetscErrorCode TaoBRGNSetL1SmoothEpsilon(Tao, PetscReal);
 PETSC_EXTERN PetscErrorCode TaoBRGNSetDictionaryMatrix(Tao, Mat);
 PETSC_EXTERN PetscErrorCode TaoBRGNGetDampingVector(Tao, Vec *);
+PETSC_EXTERN PetscErrorCode TaoBNCGSetType(Tao, TaoBNCGType);
+PETSC_EXTERN PetscErrorCode TaoBNCGGetType(Tao, TaoBNCGType *);
 
 PETSC_EXTERN PetscErrorCode TaoADMMGetMisfitSubsolver(Tao, Tao *);
 PETSC_EXTERN PetscErrorCode TaoADMMGetRegularizationSubsolver(Tao, Tao *);
@@ -482,6 +523,7 @@ PETSC_EXTERN PetscErrorCode TaoADMMSetSpectralPenalty(Tao, PetscReal);
 PETSC_EXTERN PetscErrorCode TaoGetADMMParentTao(Tao, Tao *);
 PETSC_EXTERN PetscErrorCode TaoADMMSetConstraintVectorRHS(Tao, Vec);
 PETSC_EXTERN PetscErrorCode TaoADMMSetRegularizerCoefficient(Tao, PetscReal);
+PETSC_EXTERN PetscErrorCode TaoADMMGetRegularizerCoefficient(Tao, PetscReal *);
 PETSC_EXTERN PetscErrorCode TaoADMMSetMisfitConstraintJacobian(Tao, Mat, Mat, PetscErrorCode (*)(Tao, Vec, Mat, Mat, void *), void *);
 PETSC_EXTERN PetscErrorCode TaoADMMSetRegularizerConstraintJacobian(Tao, Mat, Mat, PetscErrorCode (*)(Tao, Vec, Mat, Mat, void *), void *);
 PETSC_EXTERN PetscErrorCode TaoADMMSetRegularizerHessianRoutine(Tao, Mat, Mat, PetscErrorCode (*)(Tao, Vec, Mat, Mat, void *), void *);
@@ -504,4 +546,14 @@ PETSC_EXTERN PetscErrorCode TaoALMMGetMultipliers(Tao, Vec *);
 PETSC_EXTERN PetscErrorCode TaoALMMSetMultipliers(Tao, Vec);
 PETSC_EXTERN PetscErrorCode TaoALMMGetPrimalIS(Tao, IS *, IS *);
 PETSC_EXTERN PetscErrorCode TaoALMMGetDualIS(Tao, IS *, IS *);
-#endif
+
+PETSC_EXTERN PetscErrorCode TaoVecGetSubVec(Vec, IS, TaoSubsetType, PetscReal, Vec *);
+PETSC_EXTERN PetscErrorCode TaoMatGetSubMat(Mat, IS, Vec, TaoSubsetType, Mat *);
+PETSC_EXTERN PetscErrorCode TaoGradientNorm(Tao, Vec, NormType, PetscReal *);
+PETSC_EXTERN PetscErrorCode TaoEstimateActiveBounds(Vec, Vec, Vec, Vec, Vec, Vec, PetscReal, PetscReal *, IS *, IS *, IS *, IS *, IS *);
+PETSC_EXTERN PetscErrorCode TaoBoundStep(Vec, Vec, Vec, IS, IS, IS, PetscReal, Vec);
+PETSC_EXTERN PetscErrorCode TaoBoundSolution(Vec, Vec, Vec, PetscReal, PetscInt *, Vec);
+
+PETSC_EXTERN PetscErrorCode MatCreateSubMatrixFree(Mat, IS, IS, Mat *);
+
+#include <petsctao_deprecations.h>
