@@ -1,4 +1,3 @@
-
 static char help[] = "Linear elastiticty with dimensions using 20 node serendipity elements.\n\
 This also demonstrates use of  block\n\
 diagonal data structure.  Input arguments are:\n\
@@ -29,7 +28,7 @@ int main(int argc, char **args)
   PetscReal   norm;
 
   PetscFunctionBeginUser;
-  PetscCall(PetscInitialize(&argc, &args, (char *)0, help));
+  PetscCall(PetscInitialize(&argc, &args, NULL, help));
   PetscCall(PetscOptionsGetInt(NULL, NULL, "-m", &m, NULL));
   PetscCallMPI(MPI_Comm_rank(PETSC_COMM_WORLD, &rank));
   PetscCallMPI(MPI_Comm_size(PETSC_COMM_WORLD, &size));
@@ -52,7 +51,7 @@ int main(int argc, char **args)
   PetscCall(VecAssemblyBegin(u));
   PetscCall(VecAssemblyEnd(u));
 
-  /* Compute right-hand-side */
+  /* Compute right-hand side */
   PetscCall(MatMult(mat, u, b));
 
   /* Solve linear system */
@@ -495,8 +494,18 @@ PetscErrorCode paulintegrate20(PetscReal K[60][60])
 
 /*TEST
 
-    test:
-      args: -matconvert_type seqaij -ksp_monitor_short -ksp_rtol 1.e-2  -pc_type jacobi
+    testset:
+      args: -matconvert_type seqaij -ksp_monitor_short -ksp_rtol 1.e-2 -pc_type jacobi
       requires: x
+      output_file: output/ex10_1.out
+
+      test:
+        suffix: 1
+
+      test:
+        # Test MatMult_SeqAIJ OpenMP kernels with or without inode
+        suffix: 2
+        args: -omp_num_threads 4 -mat_no_inode {{0 1}}
+        requires: PETSC_HAVE_OPENMP_KERNELS
 
 TEST*/
