@@ -1,4 +1,3 @@
-
 /*
    This provides a matrix that consists of Mats
 */
@@ -51,7 +50,7 @@ static PetscErrorCode MatSOR_BlockMat_Symmetric(Mat A, Vec bb, PetscReal omega,
 
   PetscCall(VecSet(xx, 0.0));
   PetscCall(VecGetArray(xx, &x));
-  /* copy right hand side because it must be modified during iteration */
+  /* copy right-hand side because it must be modified during iteration */
   PetscCall(VecCopy(bb, a->workb));
   PetscCall(VecGetArrayRead(a->workb, &b));
 
@@ -76,7 +75,7 @@ static PetscErrorCode MatSOR_BlockMat_Symmetric(Mat A, Vec bb, PetscReal omega,
         PetscCall(VecPlaceArray(right, x + i * bs));
         PetscCall(MatSolve(diag[i], left, right));
 
-        /* now adjust right hand side, see MatSOR_SeqSBAIJ */
+        /* now adjust right-hand side, see MatSOR_SeqSBAIJ */
         for (j = 0; j < n; j++) {
           PetscCall(MatMultTranspose(v[j], right, left));
           PetscCall(VecPlaceArray(middle, b + idx[j] * bs));
@@ -207,8 +206,8 @@ static PetscErrorCode MatSOR_BlockMat(Mat A, Vec bb, PetscReal omega, MatSORType
 static PetscErrorCode MatSetValues_BlockMat(Mat A, PetscInt m, const PetscInt im[], PetscInt n, const PetscInt in[], const PetscScalar v[], InsertMode is)
 {
   Mat_BlockMat *a = (Mat_BlockMat *)A->data;
-  PetscInt     *rp, k, low, high, t, ii, row, nrow, i, col, l, rmax, N, lastcol = -1;
-  PetscInt     *imax = a->imax, *ai = a->i, *ailen = a->ilen;
+  PetscInt     *rp, k, low, high, t, row, nrow, i, col, l, lastcol = -1;
+  PetscInt     *ai = a->i, *ailen = a->ilen;
   PetscInt     *aj = a->j, nonew = a->nonew, bs = A->rmap->bs, brow, bcol;
   PetscInt      ridx, cidx;
   PetscBool     roworiented = a->roworiented;
@@ -223,7 +222,6 @@ static PetscErrorCode MatSetValues_BlockMat(Mat A, PetscInt m, const PetscInt im
     PetscCheck(row < A->rmap->N, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Row too large: row %" PetscInt_FMT " max %" PetscInt_FMT, row, A->rmap->N - 1);
     rp   = aj + ai[brow];
     ap   = aa + ai[brow];
-    rmax = imax[brow];
     nrow = ailen[brow];
     low  = 0;
     high = nrow;
@@ -251,7 +249,8 @@ static PetscErrorCode MatSetValues_BlockMat(Mat A, PetscInt m, const PetscInt im
         if (rp[i] == bcol) goto noinsert1;
       }
       if (nonew == 1) goto noinsert1;
-      PetscCheck(nonew != -1, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Inserting a new nonzero (%" PetscInt_FMT ", %" PetscInt_FMT ") in the matrix", row, col);
+      SETERRQ(PETSC_COMM_SELF, PETSC_ERR_SUP, "Inserting a new nonzero (%" PetscInt_FMT ", %" PetscInt_FMT ") in the block matrix", row, col);
+#if 0
       MatSeqXAIJReallocateAIJ(A, a->mbs, 1, nrow, brow, bcol, rmax, aa, ai, aj, rp, ap, imax, nonew, Mat);
       N = nrow++ - 1;
       high++;
@@ -263,7 +262,7 @@ static PetscErrorCode MatSetValues_BlockMat(Mat A, PetscInt m, const PetscInt im
       if (N >= i) ap[i] = NULL;
       rp[i] = bcol;
       a->nz++;
-      A->nonzerostate++;
+#endif
     noinsert1:;
       if (!*(ap + i)) PetscCall(MatCreateSeqAIJ(PETSC_COMM_SELF, bs, bs, 0, NULL, ap + i));
       PetscCall(MatSetValues(ap[i], 1, &ridx, 1, &cidx, &value, is));
@@ -314,7 +313,7 @@ static PetscErrorCode MatLoad_BlockMat(Mat newmat, PetscViewer viewer)
       notdone = PETSC_FALSE;
       nextcol = 1000000000;
       for (j = 0; j < bs; j++) {
-        while ((ilens[j] > 0 && ii[j][0] / bs <= currentcol)) {
+        while (ilens[j] > 0 && ii[j][0] / bs <= currentcol) {
           ii[j]++;
           ilens[j]--;
         }
@@ -332,7 +331,7 @@ static PetscErrorCode MatLoad_BlockMat(Mat newmat, PetscViewer viewer)
   if (newmat->rmap->n < 0 && newmat->rmap->N < 0 && newmat->cmap->n < 0 && newmat->cmap->N < 0) PetscCall(MatSetSizes(newmat, m, n, PETSC_DETERMINE, PETSC_DETERMINE));
   PetscCall(MatBlockMatSetPreallocation(newmat, bs, 0, lens));
   if (flg) PetscCall(MatSetOption(newmat, MAT_SYMMETRIC, PETSC_TRUE));
-  amat = (Mat_BlockMat *)(newmat)->data;
+  amat = (Mat_BlockMat *)newmat->data;
 
   /* preallocate the submatrices */
   PetscCall(PetscMalloc1(bs, &llens));
@@ -351,8 +350,8 @@ static PetscErrorCode MatLoad_BlockMat(Mat newmat, PetscViewer viewer)
       notdone = PETSC_FALSE;
       nextcol = 1000000000;
       PetscCall(PetscArrayzero(llens, bs));
-      for (j = 0; j < bs; j++) {                                /* loop over rows in block */
-        while ((ilens[j] > 0 && ii[j][0] / bs <= currentcol)) { /* loop over columns in row */
+      for (j = 0; j < bs; j++) {                              /* loop over rows in block */
+        while (ilens[j] > 0 && ii[j][0] / bs <= currentcol) { /* loop over columns in row */
           ii[j]++;
           ilens[j]--;
           llens[j]++;
@@ -460,7 +459,7 @@ static PetscErrorCode MatMult_BlockMat(Mat A, Vec x, Vec y)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode MatMult_BlockMat_Symmetric(Mat A, Vec x, Vec y)
+static PetscErrorCode MatMult_BlockMat_Symmetric(Mat A, Vec x, Vec y)
 {
   Mat_BlockMat *bmat = (Mat_BlockMat *)A->data;
   PetscScalar  *xx, *yy;
@@ -528,7 +527,7 @@ static PetscErrorCode MatMultTransposeAdd_BlockMat(Mat A, Vec x, Vec y, Vec z)
 }
 
 /*
-     Adds diagonal pointers to sparse matrix structure.
+     Adds diagonal pointers to sparse matrix nonzero structure.
 */
 static PetscErrorCode MatMarkDiagonal_BlockMat(Mat A)
 {
@@ -748,12 +747,12 @@ static struct _MatOps MatOps_Values = {MatSetValues_BlockMat,
                                        NULL,
                                        NULL,
                                        NULL,
-                                       NULL,
+                                       MatLoad_BlockMat,
                                        /* 79*/ NULL,
                                        NULL,
                                        NULL,
                                        NULL,
-                                       MatLoad_BlockMat,
+                                       NULL,
                                        /* 84*/ NULL,
                                        NULL,
                                        NULL,
@@ -814,38 +813,31 @@ static struct _MatOps MatOps_Values = {MatSetValues_BlockMat,
                                        NULL,
                                        NULL,
                                        NULL,
-                                       /*144*/ NULL,
-                                       NULL,
-                                       NULL,
-                                       NULL,
-                                       NULL,
-                                       NULL,
-                                       /*150*/ NULL,
                                        NULL};
 
 /*@C
-   MatBlockMatSetPreallocation - For good matrix assembly performance
-   the user should preallocate the matrix storage by setting the parameter nz
-   (or the array nnz).  By setting these parameters accurately, performance
-   during matrix assembly can be increased by more than a factor of 50.
+  MatBlockMatSetPreallocation - For good matrix assembly performance
+  the user should preallocate the matrix storage by setting the parameter nz
+  (or the array nnz).  By setting these parameters accurately, performance
+  during matrix assembly can be increased by more than a factor of 50.
 
-   Collective
+  Collective
 
-   Input Parameters:
-+  B - The matrix
-.  bs - size of each block in matrix
-.  nz - number of nonzeros per block row (same for all rows)
--  nnz - array containing the number of nonzeros in the various block rows
+  Input Parameters:
++ B   - The matrix
+. bs  - size of each block in matrix
+. nz  - number of nonzeros per block row (same for all rows)
+- nnz - array containing the number of nonzeros in the various block rows
          (possibly different for each row) or `NULL`
 
-   Level: intermediate
+  Level: intermediate
 
-   Notes:
-     If `nnz` is given then `nz` is ignored
+  Notes:
+  If `nnz` is given then `nz` is ignored
 
-   Specify the preallocated storage with either `nz` or `nnz` (not both).
-   Set `nz` = `PETSC_DEFAULT` and `nnz` = `NULL` for PETSc to control dynamic memory
-   allocation.
+  Specify the preallocated storage with either `nz` or `nnz` (not both).
+  Set `nz` = `PETSC_DEFAULT` and `nnz` = `NULL` for PETSc to control dynamic memory
+  allocation.
 
 .seealso: [](ch_matrices), `Mat`, `MatCreate()`, `MatCreateBlockMat()`, `MatSetValues()`
 @*/
@@ -883,25 +875,26 @@ static PetscErrorCode MatBlockMatSetPreallocation_BlockMat(Mat A, PetscInt bs, P
   PetscCall(VecCreateSeq(PETSC_COMM_SELF, bs, &bmat->left));
 
   if (!bmat->imax) PetscCall(PetscMalloc2(A->rmap->n, &bmat->imax, A->rmap->n, &bmat->ilen));
-  if (PetscLikely(nnz)) {
-    nz = 0;
-    for (i = 0; i < A->rmap->n / A->rmap->bs; i++) {
-      bmat->imax[i] = nnz[i];
-      nz += nnz[i];
-    }
-  } else SETERRQ(PETSC_COMM_SELF, PETSC_ERR_SUP, "Currently requires block row by row preallocation");
+  PetscCheck(PetscLikely(nnz), PETSC_COMM_SELF, PETSC_ERR_SUP, "Currently requires block row by row preallocation");
+  nz = 0;
+  for (i = 0; i < A->rmap->n / A->rmap->bs; i++) {
+    bmat->imax[i] = nnz[i];
+    nz += nnz[i];
+  }
 
   /* bmat->ilen will count nonzeros in each row so far. */
   PetscCall(PetscArrayzero(bmat->ilen, bmat->mbs));
 
   /* allocate the matrix space */
   PetscCall(MatSeqXAIJFreeAIJ(A, (PetscScalar **)&bmat->a, &bmat->j, &bmat->i));
-  PetscCall(PetscMalloc3(nz, &bmat->a, nz, &bmat->j, A->rmap->n + 1, &bmat->i));
+  PetscCall(PetscShmgetAllocateArray(nz, sizeof(PetscScalar), (void **)&bmat->a));
+  PetscCall(PetscShmgetAllocateArray(nz, sizeof(PetscInt), (void **)&bmat->j));
+  PetscCall(PetscShmgetAllocateArray(A->rmap->n + 1, sizeof(PetscInt), (void **)&bmat->i));
+  bmat->free_a  = PETSC_TRUE;
+  bmat->free_ij = PETSC_TRUE;
+
   bmat->i[0] = 0;
   for (i = 1; i < bmat->mbs + 1; i++) bmat->i[i] = bmat->i[i - 1] + bmat->imax[i - 1];
-  bmat->singlemalloc = PETSC_TRUE;
-  bmat->free_a       = PETSC_TRUE;
-  bmat->free_ij      = PETSC_TRUE;
 
   bmat->nz            = 0;
   bmat->maxnz         = nz;
@@ -925,9 +918,8 @@ PETSC_EXTERN PetscErrorCode MatCreate_BlockMat(Mat A)
 
   PetscFunctionBegin;
   PetscCall(PetscNew(&b));
-  A->data = (void *)b;
-  PetscCall(PetscMemcpy(A->ops, &MatOps_Values, sizeof(struct _MatOps)));
-
+  A->data         = (void *)b;
+  A->ops[0]       = MatOps_Values;
   A->assembled    = PETSC_TRUE;
   A->preallocated = PETSC_FALSE;
   PetscCall(PetscObjectChangeTypeName((PetscObject)A, MATBLOCKMAT));
@@ -937,31 +929,31 @@ PETSC_EXTERN PetscErrorCode MatCreate_BlockMat(Mat A)
 }
 
 /*@C
-   MatCreateBlockMat - Creates a new matrix in which each block contains a uniform-size sequential `Mat` object
+  MatCreateBlockMat - Creates a new matrix in which each block contains a uniform-size sequential `Mat` object
 
   Collective
 
-   Input Parameters:
-+  comm - MPI communicator
-.  m - number of rows
-.  n  - number of columns
-.  bs - size of each submatrix
-.  nz  - expected maximum number of nonzero blocks in row (use `PETSC_DEFAULT` if not known)
--  nnz - expected number of nonzers per block row if known (use `NULL` otherwise)
+  Input Parameters:
++ comm - MPI communicator
+. m    - number of rows
+. n    - number of columns
+. bs   - size of each submatrix
+. nz   - expected maximum number of nonzero blocks in row (use `PETSC_DEFAULT` if not known)
+- nnz  - expected number of nonzers per block row if known (use `NULL` otherwise)
 
-   Output Parameter:
-.  A - the matrix
+  Output Parameter:
+. A - the matrix
 
-   Level: intermediate
+  Level: intermediate
 
-   Notes:
-    Matrices of this type are nominally-sparse matrices in which each "entry" is a `Mat` object.  Each `Mat` must
-   have the same size and be sequential.  The local and global sizes must be compatible with this decomposition.
+  Notes:
+  Matrices of this type are nominally-sparse matrices in which each "entry" is a `Mat` object.  Each `Mat` must
+  have the same size and be sequential.  The local and global sizes must be compatible with this decomposition.
 
-   For matrices containing parallel submatrices and variable block sizes, see `MATNEST`.
+  For matrices containing parallel submatrices and variable block sizes, see `MATNEST`.
 
-   Developer Note:
-   I don't like the name, it is not `MATNESTMAT`
+  Developer Notes:
+  I don't like the name, it is not `MATNESTMAT`
 
 .seealso: [](ch_matrices), `Mat`, `MATBLOCKMAT`, `MatCreateNest()`
 @*/
