@@ -1,4 +1,3 @@
-
 /*
       Code for manipulating files.
 */
@@ -20,20 +19,20 @@
 #if defined(PETSC_HAVE_SYS_SYSTEMINFO_H)
   #include <sys/systeminfo.h>
 #endif
+#include <errno.h>
 
 /*@C
-   PetscGetWorkingDirectory - Gets the current working directory.
+  PetscGetWorkingDirectory - Gets the current working directory.
 
-   Not Collective
+  Not Collective
 
-   Input Parameters:
-.  len  - maximum length of path
+  Input Parameter:
+. len - maximum length of `path`
 
-   Output Parameter:
-.  path - use to hold the result value. The string should be long enough
-          to hold the path.
+  Output Parameter:
+. path - holds the result value. The string should be long enough to hold the path, for example, `PETSC_MAX_PATH_LEN`
 
-   Level: developer
+  Level: developer
 
 .seealso: `PetscGetTmp()`, `PetscSharedTmp()`, `PetscSharedWorkingDirectory()`, `PetscGetHomeDirectory()`
 @*/
@@ -41,13 +40,11 @@ PetscErrorCode PetscGetWorkingDirectory(char path[], size_t len)
 {
   PetscFunctionBegin;
 #if defined(PETSC_HAVE_GETCWD)
-  PetscCheck(getcwd(path, len), PETSC_COMM_SELF, PETSC_ERR_LIB, "getcwd()");
+  PetscCheck(getcwd(path, len), PETSC_COMM_SELF, PETSC_ERR_LIB, "Error in getcwd() due to \"%s\"", strerror(errno));
 #elif defined(PETSC_HAVE__GETCWD)
   _getcwd(path, len);
-#elif defined(PETSC_HAVE_GETWD)
-  getwd(path);
 #else
-  SETERRQ(PETSC_COMM_SELF, PETSC_ERR_SUP_SYS, "Could not find getcwd() or getwd()");
+  SETERRQ(PETSC_COMM_SELF, PETSC_ERR_SUP_SYS, "Could not find getcwd()");
 #endif
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
